@@ -36,6 +36,24 @@ fn infer_slice_method() {
     );
 }
 
+#[test]
+fn infer_array_inherent_impl() {
+    check_types(
+        r#"
+        #[lang = "array"]
+        impl<T, const N: usize> [T; N] {
+            fn foo(&self) -> T {
+                loop {}
+            }
+        }
+        fn test(x: &[u8; 0]) {
+            <[_; 0]>::foo(x);
+          //^^^^^^^^^^^^^^^^ u8
+        }
+        "#,
+    );
+}
+
 #[test]
 fn infer_associated_method_struct() {
     check_infer(
@@ -256,8 +274,8 @@ fn test() {
 } //^ i128
 
 //- /lib.rs crate:other_crate
-mod foo {
-    struct S;
+pub mod foo {
+    pub struct S;
     impl S {
         fn thing() -> i128 { 0 }
     }
@@ -922,6 +940,7 @@ fn test() { foo.call(); }
 fn super_trait_impl_return_trait_method_resolution() {
     check_infer(
         r#"
+        //- minicore: sized
         trait Base {
             fn foo(self) -> usize;
         }
@@ -1064,22 +1083,22 @@ fn dyn_trait_super_trait_not_in_scope() {
 fn method_resolution_foreign_opaque_type() {
     check_infer(
         r#"
-        extern "C" {
-            type S;
-            fn f() -> &'static S;
-        }
+extern "C" {
+    type S;
+    fn f() -> &'static S;
+}
 
-        impl S {
-            fn foo(&self) -> bool {
-                true
-            }
-        }
+impl S {
+    fn foo(&self) -> bool {
+        true
+    }
+}
 
-        fn test() {
-            let s = unsafe { f() };
-            s.foo();
-        }
-        "#,
+fn test() {
+    let s = unsafe { f() };
+    s.foo();
+}
+"#,
         expect![[r#"
             75..79 'self': &S
             89..109 '{     ...     }': bool
@@ -1087,7 +1106,7 @@ fn method_resolution_foreign_opaque_type() {
             123..167 '{     ...o(); }': ()
             133..134 's': &S
             137..151 'unsafe { f() }': &S
-            144..151 '{ f() }': &S
+            137..151 'unsafe { f() }': &S
             146..147 'f': fn f() -> &S
             146..149 'f()': &S
             157..158 's': &S
@@ -1264,6 +1283,35 @@ fn f() {
     );
 }
 
+#[test]
+fn resolve_const_generic_array_methods() {
+    check_types(
+        r#"
+#[lang = "array"]
+impl<T, const N: usize> [T; N] {
+    pub fn map<F, U>(self, f: F) -> [U; N]
+    where
+        F: FnMut(T) -> U,
+    { loop {} }
+}
+
+#[lang = "slice"]
+impl<T> [T] {
+    pub fn map<F, U>(self, f: F) -> &[U]
+    where
+        F: FnMut(T) -> U,
+    { loop {} }
+}
+
+fn f() {
+    let v = [1, 2].map::<_, usize>(|x| -> x * 2);
+    v;
+  //^ [usize; _]
+}
+    "#,
+    );
+}
+
 #[test]
 fn skip_array_during_method_dispatch() {
     check_types(
@@ -1312,3 +1360,29 @@ impl<'a, T> IntoIterator for &'a [T] {
     "#,
     );
 }
+
+#[test]
+fn sized_blanket_impl() {
+    check_infer(
+        r#"
+//- minicore: sized
+trait Foo { fn foo() -> u8; }
+impl<T: Sized> Foo for T {}
+fn f<S: Sized, T, U: ?Sized>() {
+    u32::foo;
+    S::foo;
+    T::foo;
+    U::foo;
+    <[u32]>::foo;
+}
+"#,
+        expect![[r#"
+            89..160 '{     ...foo; }': ()
+            95..103 'u32::foo': fn foo<u32>() -> u8
+            109..115 'S::foo': fn foo<S>() -> u8
+            121..127 'T::foo': fn foo<T>() -> u8
+            133..139 'U::foo': {unknown}
+            145..157 '<[u32]>::foo': {unknown}
+        "#]],
+    );
+}
